import sys, os
import numpy as np

import BioNetInput

# Data
kbt = 4.11

# System data
monomersPerPolymer = 10
radius = 2.5		
drag = 6 * np.pi * radius

stericK = 10000

# Site stuff (7 types of protein in the models)
modelProteins = []
modelProteins.append([np.array([1.0,0.0,0.0])])
modelProteins.append([np.array([-1.0,0.0,0.0]), np.array([1.0,0.0,0.0])])
modelProteins.append([np.array([-1.0,0.0,0.0])])

# Variables
models = [[0,1,1,1,1,1,1,1,1,2]]
modelNames = ["GC%d-A" % (i) for i in range(len(models))]
lOn2Rs = [0.132,0.061,0.023]
lOn2RNames = ["Flexible", "SemiFlexible", "Rigid"]

# Loop variables and build a new script for every one
lRIndex = 0
for lOn2R in lOn2Rs:

	# Calculate the new values, given that the polymer length needs to stay constant
	lPdL = lOn2R * 2 * radius
	linkerL = 0.0
	linkerDL = lPdL
	linkerK = kbt / np.power(linkerDL, 2.0)
	
	Lc = (monomersPerPolymer - 1) * (2 * radius + linkerL + linkerDL)
	
	mIndex = 0
	for model in models[:1]:
	
		# Script
		inp = BioNetInput.BioNetInput()
		
		# Params
		inp.params = BioNetInput.BioNetInputParameters()
		rouseTime = (2.0/(np.pi * kbt)) * np.power(monomersPerPolymer-1, 2) * np.power(linkerL + linkerDL + 2 * radius,2) * radius
		minimumTime = np.min([0.1, drag / (2 * linkerK)])
		inp.params.dt = minimumTime / 10.0
		inp.params.simTime = rouseTime * 400
		inp.params.timePerFrame = rouseTime
		
		# Proteins
		for mType in set(model):

			prot = BioNetInput.BioNetInputProtein()
			prot.name = "Protein" + chr(65 + mType)
			prot.type = "sphere"
			prot.radius = radius
			
			# Protein addition
			prot.maxNumConnections = len(modelProteins[mType])
			prot.numSites = len(modelProteins[mType])
			prot.site = []

			# Sites
			mpIndex = 0
			for mp in modelProteins[mType]:
				sit = BioNetInput.BioNetInputSite()
				sit.name = "Site" + chr(65 + mpIndex)
				sit.type = "point"
				sit.position = mp

				prot.site.append(sit)
				mpIndex += 1

			inp.protein.append(prot)
			inp.numProteins += 1
			
		# Bonds (we only need linker here, not crosslink)
		bon = BioNetInput.BioNetInputBond()
		bon.name = "Linker"
		bon.type = "spring"
		bon.k = linkerK
		bon.l = linkerL

		inp.bond.append(bon)
		inp.numBonds += 1
		
		# Nodes
		for nIndex in range(monomersPerPolymer):
			
			nod = BioNetInput.BioNetInputNode()
			
			for p in inp.protein:
				if p.name == "Protein" + chr(65 + model[nIndex]):
					nod.protein = p
					break
			
			nod.pos = np.array([nIndex * (2 * radius + linkerL),0.0,0.0])
			nod.vel = np.array([0.0,0.0,0.0])
			nod.orientation = np.array([0.0,0.0,0.0])

			inp.node.append(nod)
			inp.numNodes += 1
			
		# Connections
		for cIndex in range(monomersPerPolymer - 1):
			
			con = BioNetInput.BioNetInputConnection()
			con.bond = inp.bond[0]
			con.nIndex = [cIndex, cIndex+1]
			con.sIndex = [inp.node[con.nIndex[0]].protein.numSites - 1, 0]
			
			inp.connection.append(con)
			inp.numConnections += 1
			
		# Box
		inp.box = BioNetInput.BioNetInputSimulationBox()
		inp.box.BC = "HBC"
		inp.box.dimension = np.array([1.0,1.0,1.0]) * 10.0 * Lc
		inp.box.numVoxels = [1,1,1]
		
		# Forces (steric)
		force = BioNetInput.BioNetInputForce()
		force.name = "steric"
		force.attributes = {"k":stericK}
		
		inp.force.append(force)
		inp.numForces += 1
		
		# Write
		oFname = "../input/SinglePolymer_Model%s_%s.xml" % (modelNames[mIndex], lOn2RNames[lRIndex])
		inp.writeToFile(oFname)
		
		mIndex += 1
	lRIndex += 1
